# Generated by Django 5.0.4 on 2024-04-12 01:42

import django.core.validators
import edc_model_fields.fields.custom_django_fields
import edc_model_fields.fields.other_charfield
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_subject", "0165_careseekinga_care_visit_reason_other_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="careseekinga",
            name="med_collected_location",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                choices=[
                    ("public_pharmacy", "Public pharmacy"),
                    ("private_pharmacy", "Private pharmacy"),
                    ("club", "Club"),
                    ("OTHER", "Other, please specify ..."),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                metadata="FMEDCOLLECTLOC1",
                verbose_name="Where did you collect the medicines from?",
            ),
        ),
        migrations.AddField(
            model_name="careseekinga",
            name="med_collected_location_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                metadata="FMEDCOLLECTLOCOTHER1",
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicalcareseekinga",
            name="med_collected_location",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                choices=[
                    ("public_pharmacy", "Public pharmacy"),
                    ("private_pharmacy", "Private pharmacy"),
                    ("club", "Club"),
                    ("OTHER", "Other, please specify ..."),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                metadata="FMEDCOLLECTLOC1",
                verbose_name="Where did you collect the medicines from?",
            ),
        ),
        migrations.AddField(
            model_name="historicalcareseekinga",
            name="med_collected_location_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                metadata="FMEDCOLLECTLOCOTHER1",
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AlterField(
            model_name="careseekinga",
            name="care_visit_duration",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                help_text="Invalid format. Please insert a numeric values followed by “h” for hours, and a numeric values followed by “m” for minutes. For example, 1h2m, 0h35m, and so on",
                max_length=5,
                metadata="FFACTIME1",
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How much time did you spend during your visit today -- from arrival to this place until the end of your visit?",
            ),
        ),
        migrations.AlterField(
            model_name="careseekinga",
            name="med_collected",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=25,
                metadata="FMEDCOLL1",
                verbose_name="Did you receive/collect these medicines (whether paid or received for free)?",
            ),
        ),
        migrations.AlterField(
            model_name="careseekinga",
            name="med_cost_dm",
            field=edc_model_fields.fields.custom_django_fields.IntegerField2(
                blank=True,
                help_text="Leave blank if not applicable. In local currency. If medicines were free enter `0`.",
                metadata="FMEDCOSTDM1",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(9999999),
                ],
                verbose_name="How much was spent on Diabetes medicines? ",
            ),
        ),
        migrations.AlterField(
            model_name="careseekinga",
            name="med_cost_hiv",
            field=edc_model_fields.fields.custom_django_fields.IntegerField2(
                blank=True,
                help_text="Leave blank if not applicable. In local currency. If medicines were free enter `0`.",
                metadata="FMEDCOSTHIV1",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(9999999),
                ],
                verbose_name="How much was spent on HIV medicines? ",
            ),
        ),
        migrations.AlterField(
            model_name="careseekinga",
            name="med_cost_htn",
            field=edc_model_fields.fields.custom_django_fields.IntegerField2(
                blank=True,
                help_text="Leave blank if not applicable. In local currency. If medicines were free enter `0`.",
                metadata="FMEDCOSTHTN1",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(9999999),
                ],
                verbose_name="How much was spent on Hypertension medicines? ",
            ),
        ),
        migrations.AlterField(
            model_name="careseekinga",
            name="med_cost_other",
            field=edc_model_fields.fields.custom_django_fields.IntegerField2(
                blank=True,
                help_text="Leave blank if not applicable. In local currency. If medicines were free enter `0`.",
                metadata="FMEDCOSTOTHER1",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(9999999),
                ],
                verbose_name="How much was spent on OTHER medicines? ",
            ),
        ),
        migrations.AlterField(
            model_name="careseekinga",
            name="med_cost_tot",
            field=edc_model_fields.fields.custom_django_fields.IntegerField2(
                blank=True,
                help_text="Leave blank if not applicable. In local currency. If medicines were free enter `0`.",
                metadata="FMEDCOST1",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(9999999),
                ],
                verbose_name="How much was spent on these medicines? ",
            ),
        ),
        migrations.AlterField(
            model_name="careseekinga",
            name="travel_time",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                help_text="Invalid format. Please insert a numeric values followed by “h” for hours, and a numeric values followed by “m” for minutes. For example, 1h2m, 0h35m, and so on",
                max_length=5,
                metadata="FTRATIME1",
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How long did it take you to reach here?",
            ),
        ),
        migrations.AlterField(
            model_name="careseekinga",
            name="wait_duration",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                help_text="Invalid format. Please insert a numeric values followed by “h” for hours, and a numeric values followed by “m” for minutes. For example, 1h2m, 0h35m, and so on",
                max_length=5,
                metadata="FWAITIME1",
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How much time did you spend waiting?",
            ),
        ),
        migrations.AlterField(
            model_name="careseekinga",
            name="with_hcw_duration",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                help_text="Invalid format. Please insert a numeric values followed by “h” for hours, and a numeric values followed by “m” for minutes. For example, 1h2m, 0h35m, and so on",
                max_length=5,
                metadata="FWORKTIME1",
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How much time did you spend with the healthcare worker?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekinga",
            name="care_visit_duration",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                help_text="Invalid format. Please insert a numeric values followed by “h” for hours, and a numeric values followed by “m” for minutes. For example, 1h2m, 0h35m, and so on",
                max_length=5,
                metadata="FFACTIME1",
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How much time did you spend during your visit today -- from arrival to this place until the end of your visit?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekinga",
            name="med_collected",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=25,
                metadata="FMEDCOLL1",
                verbose_name="Did you receive/collect these medicines (whether paid or received for free)?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekinga",
            name="med_cost_dm",
            field=edc_model_fields.fields.custom_django_fields.IntegerField2(
                blank=True,
                help_text="Leave blank if not applicable. In local currency. If medicines were free enter `0`.",
                metadata="FMEDCOSTDM1",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(9999999),
                ],
                verbose_name="How much was spent on Diabetes medicines? ",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekinga",
            name="med_cost_hiv",
            field=edc_model_fields.fields.custom_django_fields.IntegerField2(
                blank=True,
                help_text="Leave blank if not applicable. In local currency. If medicines were free enter `0`.",
                metadata="FMEDCOSTHIV1",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(9999999),
                ],
                verbose_name="How much was spent on HIV medicines? ",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekinga",
            name="med_cost_htn",
            field=edc_model_fields.fields.custom_django_fields.IntegerField2(
                blank=True,
                help_text="Leave blank if not applicable. In local currency. If medicines were free enter `0`.",
                metadata="FMEDCOSTHTN1",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(9999999),
                ],
                verbose_name="How much was spent on Hypertension medicines? ",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekinga",
            name="med_cost_other",
            field=edc_model_fields.fields.custom_django_fields.IntegerField2(
                blank=True,
                help_text="Leave blank if not applicable. In local currency. If medicines were free enter `0`.",
                metadata="FMEDCOSTOTHER1",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(9999999),
                ],
                verbose_name="How much was spent on OTHER medicines? ",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekinga",
            name="med_cost_tot",
            field=edc_model_fields.fields.custom_django_fields.IntegerField2(
                blank=True,
                help_text="Leave blank if not applicable. In local currency. If medicines were free enter `0`.",
                metadata="FMEDCOST1",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(9999999),
                ],
                verbose_name="How much was spent on these medicines? ",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekinga",
            name="travel_time",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                help_text="Invalid format. Please insert a numeric values followed by “h” for hours, and a numeric values followed by “m” for minutes. For example, 1h2m, 0h35m, and so on",
                max_length=5,
                metadata="FTRATIME1",
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How long did it take you to reach here?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekinga",
            name="wait_duration",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                help_text="Invalid format. Please insert a numeric values followed by “h” for hours, and a numeric values followed by “m” for minutes. For example, 1h2m, 0h35m, and so on",
                max_length=5,
                metadata="FWAITIME1",
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How much time did you spend waiting?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekinga",
            name="with_hcw_duration",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                help_text="Invalid format. Please insert a numeric values followed by “h” for hours, and a numeric values followed by “m” for minutes. For example, 1h2m, 0h35m, and so on",
                max_length=5,
                metadata="FWORKTIME1",
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How much time did you spend with the healthcare worker?",
            ),
        ),
    ]



import re
from decimal import Decimal
from uuid import uuid4

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.db import IntegrityError
from django.test import TestCase, override_settings, tag
from edc_constants.constants import (
    DM,
    FEMALE,
    HIV,
    HTN,
    MALE,
    NO,
    TBD,
    UUID_PATTERN,
    YES,
)
from intecomm_form_validators import RECRUITING
from model_bakery.baker import make_recipe

from intecomm_lists.models import Conditions, ScreeningRefusalReasons
from intecomm_screening.forms import PatientLogForm, SubjectScreeningForm
from intecomm_screening.models import PatientLog, SubjectScreening
from intecomm_screening.models.subject_screening import SubjectScreeningError
from intecomm_screening.utils import InvalidScreeningIdentifier

from ..intecomm_test_case_mixin import IntecommTestCaseMixin


class TestScreening(IntecommTestCaseMixin, TestCase):
    def setUp(self) -> None:
        super().setUp()
        self.user = User.objects.create_superuser("user_login", "u@example.com", "pass")
        self.user.is_active = True
        self.user.is_staff = True
        self.user.is_superuser = True
        self.user.save()
        self.user.refresh_from_db()
        self.user.userprofile.sites.add(Site.objects.get(id=101))

    def test_no_patients_ok(self):
        obj = make_recipe("intecomm_screening.patientgroup")
        self.assertEqual(obj.status, RECRUITING)
        self.assertIsNotNone(obj.name)
        self.assertIsNotNone(obj.group_identifier)
        self.assertTrue(re.match(UUID_PATTERN, str(obj.group_identifier)))
        self.assertEqual(obj.ratio, Decimal("0"))
        self.assertEqual(obj.randomize_now, NO)
        self.assertFalse(obj.randomized)
        self.assertEqual(obj.patients.all().count(), 0)

    def test_with_patients_ok(self):
        obj = None
        initials = "ABCDEFGHIJKLMNOP"
        for i in range(0, 4):
            obj = make_recipe(
                "intecomm_screening.patientlog",
                legal_name=f"NAMEA{i} AAA{i}",
                familiar_name=f"NAMEA{i}",
                initials=f"N{initials[i]}A",
                hospital_identifier=uuid4().hex,
                contact_number=f"123456789{i}",
                site_id=settings.SITE_ID,
            )
            obj.conditions.add(Conditions.objects.get(name=HIV))
        for i in range(0, 5):
            make_recipe(
                "intecomm_screening.patientlog",
                legal_name=f"NAMEB{i} BBB{i}",
                familiar_name=f"NAMEB{i}",
                hospital_identifier=uuid4().hex,
                contact_number=f"12345678{i}9",
                site_id=settings.SITE_ID,
            )
            obj.conditions.add(Conditions.objects.get(name=DM))
        for i in range(0, 5):
            make_recipe(
                "intecomm_screening.patientlog",
                legal_name=f"NAMEC{i} CCC{i}",
                familiar_name=f"NAMEC{i}",
                hospital_identifier=uuid4().hex,
                contact_number=f"1234567{i}89",
                site_id=settings.SITE_ID,
            )
            obj.conditions.add(Conditions.objects.get(name=HTN))
        for patient_log in PatientLog.objects.all():
            self.assertEqual(patient_log.patientgroup_set.all().count(), 0)
        self.assertEqual(PatientLog.objects.all().count(), 14)

        # create patient_group
        patient_group = make_recipe("intecomm_screening.patientgroup")

        # add patient_logs to patient_group
        for index, patient_log in enumerate(PatientLog.objects.all()):
            patient_group.patients.add(patient_log)
        for patient_log in PatientLog.objects.all():
            self.assertEqual(patient_log.patientgroup_set.all().count(), 1)
            self.assertEqual(patient_log.patientgroup_set.all().first(), patient_group)

        patient_group.refresh_from_db()

        # TODO: why does this fail?
        # self.assertEqual(patient_group.patients.count(), 14)

    def test_screening_raises_if_unwilling_in_patient_log(self):
        patient_log = make_recipe(
            "intecomm_screening.patientlog",
            legal_name="NAMEA AAA",
            familiar_name="NAMEA",
            initials="NA",
            hospital_identifier=uuid4().hex,
            contact_number="1234567890",
            willing_to_screen=NO,
            screening_refusal_reason=ScreeningRefusalReasons.objects.get(
                name="dont_have_time"
            ),
            site_id=settings.SITE_ID,
        )
        patient_log.conditions.add(Conditions.objects.get(name=HIV))
        subject_screening = SubjectScreening(
            patient_log_identifier=patient_log.patient_log_identifier,
            hospital_identifier=patient_log.hospital_identifier,
            gender=patient_log.gender,
            age_in_years=patient_log.age_in_years,
            legal_name=patient_log.legal_name,
            familiar_name=patient_log.familiar_name,
            initials=patient_log.initials,
        )

        with self.assertRaises(SubjectScreeningError) as cm:
            subject_screening.save()
        self.assertIn(
            f"Patient '{patient_log.patient_log_identifier}' is unwilling to screen.",
            str(cm.exception),
        )

    def test_screening_ok_if_not_unwilling_in_patient_log(self):
        patient_log = make_recipe(
            "intecomm_screening.patientlog",
            legal_name="NAMEA AAA",
            familiar_name="NAMEA",
            initials="NA",
            hospital_identifier=uuid4().hex,
            contact_number="1234567890",
            willing_to_screen=YES,
            screening_refusal_reason=None,
            site_id=settings.SITE_ID,
        )
        patient_log.conditions.add(Conditions.objects.get(name=HIV))
        subject_screening = SubjectScreening(
            patient_log_identifier=patient_log.patient_log_identifier,
            hospital_identifier=patient_log.hospital_identifier,
            gender=patient_log.gender,
            age_in_years=patient_log.age_in_years,
            legal_name=patient_log.legal_name,
            familiar_name=patient_log.familiar_name,
            initials=patient_log.initials,
        )

        try:
            subject_screening.save()
        except SubjectScreeningError as e:
            self.fail(f"SubjectScreeningError unexpectedly raised. Got {e}")

    def test_screening_raises_if_hospital_identifier_mismatch(self):
        patient_log = make_recipe(
            "intecomm_screening.patientlog",
            legal_name="NAMEA AAA",
            familiar_name="NAMEA",
            initials="NA",
            hospital_identifier="56d2c5ebc7384309990ddc14ee2cf1b2",
            contact_number="1234567890",
            site_id=settings.SITE_ID,
        )
        patient_log.conditions.add(Conditions.objects.get(name=HIV))
        subject_screening = SubjectScreening(
            patient_log_identifier=patient_log.patient_log_identifier,
            hospital_identifier="e8da3474883445cd906697d91e52a10a",
            gender=patient_log.gender,
            age_in_years=patient_log.age_in_years,
            legal_name=patient_log.legal_name,
            familiar_name=patient_log.familiar_name,
            initials=patient_log.initials,
        )

        with self.assertRaises(SubjectScreeningError) as cm:
            subject_screening.save()
        self.assertIn(
            "Health facility identifier does not match patient log. ",
            str(cm.exception),
        )

    def test_screening_raises_if_initials_mismatch(self):
        patient_log = make_recipe(
            "intecomm_screening.patientlog",
            legal_name="NAMEA AAA",
            familiar_name="NAMEA",
            initials="NA",
            hospital_identifier=uuid4().hex,
            contact_number="1234567890",
            site_id=settings.SITE_ID,
        )
        patient_log.conditions.add(Conditions.objects.get(name=HIV))
        subject_screening = SubjectScreening(
            patient_log_identifier=patient_log.patient_log_identifier,
            hospital_identifier=patient_log.hospital_identifier,
            gender=patient_log.gender,
            age_in_years=patient_log.age_in_years,
            legal_name=patient_log.legal_name,
            familiar_name=patient_log.familiar_name,
            initials="XX",
        )

        with self.assertRaises(SubjectScreeningError) as cm:
            subject_screening.save()
        self.assertIn(
            "Initials do not match patient log. ",
            str(cm.exception),
        )

    def test_patient_log_gender_cannot_be_changed_after_screen(self):
        patient_log = self.get_patient_log(willing_to_screen=YES, gender=FEMALE)
        self.get_subject_screening(patient_log=patient_log, gender=FEMALE)
        form = PatientLogForm(data=dict(gender=MALE), instance=patient_log)
        form.is_valid()
        self.assertIn("Patient has already screened. Gender", str(form._errors.get("__all__")))

    def test_patient_log_initials_cannot_be_changed_after_screen(self):
        patient_log = self.get_patient_log(willing_to_screen=YES, gender=FEMALE, initials="YY")
        self.get_subject_screening(patient_log=patient_log, gender=FEMALE)
        form = PatientLogForm(data=dict(gender=FEMALE, initials="XX"), instance=patient_log)
        form.is_valid()
        self.assertIn(
            "Patient has already screened. Initials", str(form._errors.get("__all__"))
        )

    def test_screening_id_not_found_in_patient_log_raises(self):
        patient_log = self.get_patient_log()
        self.get_subject_screening(patient_log=patient_log)
        patient_log.screening_identifier = "some_other_identifier"
        self.assertRaises(InvalidScreeningIdentifier, patient_log.save)

    def test_patient_log_screening_identifier_unique_constraint(self):
        patient_log = self.get_patient_log()
        self.get_subject_screening(patient_log=patient_log)
        patient_log_two = self.get_patient_log(legal_name="NOTHER AME")
        patient_log_two.screening_identifier = patient_log.screening_identifier
        self.assertRaises(IntegrityError, patient_log_two.save)

    @override_settings(SITE_ID=101)
    def test_unwilling_to_screen_in_patient_log_raises(self):
        patient_log = self.get_patient_log(
            willing_to_screen=NO,
            screening_refusal_reason=ScreeningRefusalReasons.objects.get(
                name="dont_have_time"
            ),
            gender=FEMALE,
        )

        cleaned_data = dict(
            patient_log_identifier=patient_log.patient_log_identifier,
            gender=FEMALE,
        )
        form = SubjectScreeningForm(data=cleaned_data, instance=SubjectScreening())
        form.is_valid()
        self.assertIn("Patient is unwilling to screen", str(form._errors.get("__all__", "")))

        patient_log.willing_to_screen = YES
        patient_log.screening_refusal_reason = None
        patient_log.save()
        form = SubjectScreeningForm(data=cleaned_data, instance=SubjectScreening())
        form.is_valid()
        self.assertNotIn(
            "Patient is unwilling to screen", str(form._errors.get("__all__", ""))
        )

    @override_settings(SITE_ID=101)
    def test_gender(self):
        patient_log = self.get_patient_log(
            gender=FEMALE,
            willing_to_screen=YES,
            screening_refusal_reason=None,
        )
        cleaned_data = dict(
            gender=MALE, patient_log_identifier=patient_log.patient_log_identifier
        )
        form = SubjectScreeningForm(data=cleaned_data, instance=SubjectScreening())
        form.is_valid()
        self.assertIn("Invalid. Expected Female", str(form._errors.get("gender", "")))

        cleaned_data = dict(
            gender=FEMALE, patient_log_identifier=patient_log.patient_log_identifier
        )
        form = SubjectScreeningForm(data=cleaned_data, instance=SubjectScreening())
        form.is_valid()
        self.assertNotIn("gender", form._errors)

    @override_settings(SITE_ID=101)
    def test_initials(self):
        patient_log = self.get_patient_log(
            gender=FEMALE,
            initials="XX",
            willing_to_screen=YES,
            screening_refusal_reason=None,
        )
        cleaned_data = dict(
            patient_log_identifier=patient_log.patient_log_identifier,
            gender=FEMALE,
            initials="ZZ",
        )
        form = SubjectScreeningForm(data=cleaned_data, instance=SubjectScreening())
        form.is_valid()
        self.assertIn("Invalid. Expected XX", str(form._errors.get("initials", "")))

        cleaned_data = dict(
            patient_log_identifier=patient_log.patient_log_identifier,
            gender=FEMALE,
            initials="XX",
        )
        form = SubjectScreeningForm(data=cleaned_data, instance=SubjectScreening())
        form.is_valid()
        self.assertNotIn("initials", form._errors)

    @tag("1")
    @override_settings(SITE_ID=101)
    def test_conditions_matching_patient_log(self):
        patient_log = self.get_patient_log(
            gender=MALE,
            age_in_years=20,
            initials="XX",
            willing_to_screen=YES,
            screening_refusal_reason=None,
            hospital_identifier="123456789",
            conditions=[DM],
            site=Site.objects.get(id=settings.SITE_ID),
        )
        cleaned_data = dict(
            report_datetime=patient_log.report_datetime,
            patient_log_identifier=patient_log.patient_log_identifier,
            gender=MALE,
            age_in_years=20,
            initials="XX",
            hospital_identifier="123456789",
            consent_ability=YES,
            in_care_6m=YES,
            in_care_duration="5y",
            hiv_dx=NO,
            dm_dx=YES,
            htn_dx=YES,
            site=Site.objects.get(id=settings.SITE_ID),
        )
        form = SubjectScreeningForm(data=cleaned_data, instance=SubjectScreening())
        form.is_valid()
        self.assertIn("Invalid. Condition not indicated", str(form._errors.get("htn_dx", "")))

        patient_log.conditions.add(Conditions.objects.get(name=HTN))
        patient_log.save()
        form = SubjectScreeningForm(data=cleaned_data, instance=SubjectScreening())
        form.is_valid()
        self.assertNotIn(
            "Invalid. Condition not indicated", str(form._errors.get("hiv_dx", ""))
        )
        self.assertNotIn(
            "Invalid. Condition not indicated", str(form._errors.get("htn_dx", ""))
        )
        self.assertNotIn(
            "Invalid. Condition not indicated", str(form._errors.get("dm_dx", ""))
        )

        patient_log.conditions.remove(Conditions.objects.get(name=HIV))
        patient_log.conditions.remove(Conditions.objects.get(name=HTN))
        patient_log.conditions.remove(Conditions.objects.get(name=DM))
        patient_log.save()
        cleaned_data.update(hiv_dx=NO, dm_dx=NO, htn_dx=NO)
        form = SubjectScreeningForm(data=cleaned_data, instance=SubjectScreening())
        form.is_valid()
        self.assertIn(
            "No conditions (HIV/DM/HTN) have been indicated for this patient",
            str(form._errors.get("__all__", "")),
        )

    @override_settings(SITE_ID=101)
    def test_health_talk_reponse_from_patientlog(self):
        patient_log = self.get_patient_log(
            gender=MALE,
            age_in_years=20,
            initials="XX",
            willing_to_screen=YES,
            screening_refusal_reason=None,
            conditions=[DM],
            first_health_talk=TBD,
            second_health_talk=TBD,
        )
        cleaned_data = dict(
            report_datetime=patient_log.report_datetime,
            patient_log_identifier=patient_log.patient_log_identifier,
            gender=MALE,
            age_in_years=20,
            initials="XX",
            consent_ability=YES,
            in_care_6m=YES,
            in_care_duration="5y",
            hiv_dx=NO,
            dm_dx=YES,
            htn_dx=NO,
        )
        form = SubjectScreeningForm(data=cleaned_data, instance=SubjectScreening())
        form.is_valid()
        self.assertIn(
            "Has patient attended the first health talk", str(form._errors.get("__all__", ""))
        )

        patient_log.first_health_talk = YES
        patient_log.second_health_talk = TBD
        patient_log.save()
        patient_log.refresh_from_db()

        form = SubjectScreeningForm(data=cleaned_data, instance=SubjectScreening())
        form.is_valid()
        self.assertIn(
            "Has patient attended the second health talk", str(form._errors.get("__all__", ""))
        )

        patient_log.first_health_talk = NO
        patient_log.second_health_talk = NO
        patient_log.save()
        patient_log.refresh_from_db()
        form = SubjectScreeningForm(data=cleaned_data, instance=SubjectScreening())
        form.is_valid()

        self.assertNotIn(
            "Has patient attended the first health talk", str(form._errors.get("__all__", ""))
        )
        self.assertNotIn(
            "Has patient attended the second health talk", str(form._errors.get("__all__", ""))
        )



# Generated by Django 4.2.3 on 2023-07-12 23:24

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0054_remove_historicalhtnreview_dia_blood_pressure_and_more"),
    ]

    operations = [
        migrations.RenameField(
            model_name="drugrefilldm",
            old_name="return_in_days",
            new_name="rx_days",
        ),
        migrations.RenameField(
            model_name="drugrefillhiv",
            old_name="return_in_days",
            new_name="rx_days",
        ),
        migrations.RenameField(
            model_name="drugrefillhtn",
            old_name="return_in_days",
            new_name="rx_days",
        ),
        migrations.RenameField(
            model_name="historicaldrugrefilldm",
            old_name="return_in_days",
            new_name="rx_days",
        ),
        migrations.RenameField(
            model_name="historicaldrugrefillhiv",
            old_name="return_in_days",
            new_name="rx_days",
        ),
        migrations.RenameField(
            model_name="historicaldrugrefillhtn",
            old_name="return_in_days",
            new_name="rx_days",
        ),
    ]



from django import forms
from edc_form_validators import FormValidatorMixin
from edc_screening.modelform_mixins import AlreadyConsentedFormMixin
from edc_sites.modelform_mixins import SiteModelFormMixin
from intecomm_form_validators import PatientLogFormValidator

from ...models import PatientLogUg
from .patient_log_form import PatientLogFormMixin


class PatientLogUgForm(
    PatientLogFormMixin,
    AlreadyConsentedFormMixin,
    SiteModelFormMixin,
    FormValidatorMixin,
    forms.ModelForm,
):
    form_validator_cls = PatientLogFormValidator

    class Meta:
        model = PatientLogUg
        fields = "__all__"
        exclude = ["legal_name", "familiar_name"]



# Generated by Django 4.2.1 on 2023-07-06 02:18

import django.db.models.manager
import edc_sites.models
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0044_alter_historicalpatientlog_site_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="consentrefusal",
            options={
                "default_manager_name": "objects",
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Consent Refusal",
                "verbose_name_plural": "Consent Refusals",
            },
        ),
        migrations.AlterModelOptions(
            name="patientcall",
            options={
                "default_manager_name": "objects",
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ["report_datetime"],
                "verbose_name": "Patient Call",
                "verbose_name_plural": "Patient Calls",
            },
        ),
        migrations.AlterModelOptions(
            name="patientlogreportprinthistory",
            options={
                "default_manager_name": "objects",
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Patient Log Report Print History",
                "verbose_name_plural": "Patient Log Report Print History",
            },
        ),
        migrations.AlterModelOptions(
            name="subjectscreening",
            options={
                "default_manager_name": "objects",
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Subject Screening",
                "verbose_name_plural": "Subject Screening",
            },
        ),
        migrations.AlterModelManagers(
            name="consentrefusal",
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                ("objects", django.db.models.manager.Manager()),
            ],
        ),
    ]



from django import forms
from intecomm_form_validators.subject import MedicationsFormValidator

from ..models import Medications
from .mixins import CrfModelFormMixin


class MedicationsForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = MedicationsFormValidator

    class Meta:
        model = Medications
        fields = "__all__"



from django.conf import settings
from edc_lab import LabProfile, RequisitionPanelGroup
from edc_lab_panel.panels import (
    blood_glucose_panel,
    cd4_panel,
    fbc_panel,
    hba1c_panel,
    lft_panel,
    lipids_panel,
    rft_panel,
    vl_panel,
)

chemistry_panel = RequisitionPanelGroup(
    lft_panel,
    rft_panel,
    lipids_panel,
    name="chemistry",
    verbose_name="Chemistry: LFT, RFT, Lipids",
    abbreviation="CHEM",
    reference_range_collection_name="intecomm",
)

subject_lab_profile = LabProfile(
    name="subject_lab_profile",
    requisition_model=settings.SUBJECT_REQUISITION_MODEL,
    reference_range_collection_name="intecomm",
)

subject_lab_profile.add_panel(blood_glucose_panel)
subject_lab_profile.add_panel(fbc_panel)
subject_lab_profile.add_panel(hba1c_panel)
subject_lab_profile.add_panel(lft_panel)
subject_lab_profile.add_panel(lipids_panel)
subject_lab_profile.add_panel(rft_panel)
subject_lab_profile.add_panel(vl_panel)
subject_lab_profile.add_panel(cd4_panel)



def get_first_fieldset():
    return (
        None,
        {
            "fields": (
                "report_datetime",
                "site",
                "patient_log_identifier",
            )
        },
    )


def get_demographics_fieldset(include_pii=None):
    fields = []
    if include_pii:
        fields.extend(["legal_name", "familiar_name"])
    fields.extend(
        [
            "initials",
            "hospital_identifier",
            "gender",
            "age_in_years",
        ]
    )
    return (
        "Demographics",
        {
            "description": (
                "Please review carefully. If anything needs to be changed, do so on "
                "the Patient Log and try again"
            ),
            "fields": tuple(fields),
        },
    )


def get_health_facility_fieldset():
    return (
        "Health facility",
        {
            "fields": (
                "in_care_6m",
                "in_care_duration",
            )
        },
    )


def get_hiv_fieldset():
    return (
        "HIV",
        {
            "fields": (
                "hiv_dx",
                "hiv_dx_6m",
                "hiv_dx_ago",
                "art_unchanged_3m",
                "art_stable",
                "art_adherent",
            )
        },
    )


def get_dm_fieldset():
    return (
        "Diabetes",
        {
            "fields": (
                "dm_dx",
                "dm_dx_6m",
                "dm_dx_ago",
                "dm_complications",
            )
        },
    )


def get_htn_fieldset():
    return (
        "Hypertension",
        {
            "fields": (
                "htn_dx",
                "htn_dx_6m",
                "htn_dx_ago",
                "htn_complications",
            )
        },
    )


def get_pregnancy_fieldset():
    return (
        "Pregnancy",
        {"fields": ("pregnant",)},
    )


def get_other_history_fieldset():
    return (
        "Other history",
        {
            "fields": (
                "excluded_by_bp_history",
                "excluded_by_gluc_history",
                "requires_acute_care",
            )
        },
    )


def get_location_fieldset():
    return (
        "Location",
        {
            "fields": (
                "lives_nearby",
                "staying_nearby_6",
            )
        },
    )


def get_bp_fieldset():
    return (
        "Blood pressure measurements",
        {
            "fields": (
                "sys_blood_pressure_one",
                "dia_blood_pressure_one",
                "sys_blood_pressure_two",
                "dia_blood_pressure_two",
            )
        },
    )


def get_other_fieldset():
    return (
        "Other",
        {
            "fields": (
                "consent_ability",
                "unsuitable_for_study",
                "reasons_unsuitable",
                "unsuitable_agreed",
            )
        },
    )


def get_updates_fieldset():
    return (
        "Updates",
        {
            "classes": ("collapse",),
            "fields": (
                "screening_identifier",
                "eligible",
                "eligibility_datetime",
                "real_eligibility_datetime",
                "reasons_ineligible",
                "subject_identifier",
            ),
        },
    )



from django.core.validators import MaxValueValidator, MinValueValidator
from django.utils.translation import gettext as _
from edc_model_fields.fields import IntegerField2


class ExpenseField(IntegerField2):
    def __init__(self, *args, **kwargs):
        self.custom_null = kwargs.get("null")
        self.custom_blank = kwargs.get("blank")
        if self.custom_null is None:
            kwargs.update(null=True)
        if self.custom_blank is None:
            kwargs.update(blank=True)
        kwargs.update(validators=[MinValueValidator(0), MaxValueValidator(9999999)])
        if kwargs.get("blank"):
            default_help_text = _(
                "In local currency. If nothing spent enter `0` or leave blank "
                "if a response is not required."
            )
        else:
            default_help_text = _("In local currency. If nothing spent enter `0`.")
        self.custom_help_text = kwargs.get("help_text")
        kwargs.update(help_text=self.custom_help_text or default_help_text)
        super().__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        del kwargs["validators"]
        if self.custom_help_text is None:
            del kwargs["help_text"]
        if self.custom_null is None:
            del kwargs["null"]
        if self.custom_blank is None:
            del kwargs["blank"]
        return name, path, args, kwargs



# Generated by Django 5.0.4 on 2024-04-10 21:49

import edc_model_fields.fields.custom_django_fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_subject", "0159_careseekingb_missed_activities_other_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="careseekingb",
            name="inpatient",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Were you admitted as a INPATIENT?",
            ),
        ),
        migrations.AddField(
            model_name="historicalcareseekingb",
            name="inpatient",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Were you admitted as a INPATIENT?",
            ),
        ),
        migrations.AlterField(
            model_name="careseekingb",
            name="accompany_alt",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                choices=[
                    (
                        "paid_work",
                        "Paid work (e.g. full-time employment, small business owners/traders, day jobs, etc.)",
                    ),
                    ("unpaid_work", "Unpaid work (e.g. subsistence farming, housework etc.)"),
                    ("OTHER", "Other (specify)"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                metadata="FOUTACMPACT1",
                verbose_name="If those accompanying you were not attending your last/most recent visit with you, what would they have been doing?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekingb",
            name="accompany_alt",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                choices=[
                    (
                        "paid_work",
                        "Paid work (e.g. full-time employment, small business owners/traders, day jobs, etc.)",
                    ),
                    ("unpaid_work", "Unpaid work (e.g. subsistence farming, housework etc.)"),
                    ("OTHER", "Other (specify)"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                metadata="FOUTACMPACT1",
                verbose_name="If those accompanying you were not attending your last/most recent visit with you, what would they have been doing?",
            ),
        ),
    ]



from edc_constants.constants import NOT_APPLICABLE, OTHER

RESPONDENT_CHOICES = (
    ("patient", "Patient"),
    ("family", "Family"),
    ("friend", "friend"),
    (OTHER, "other"),
    (NOT_APPLICABLE, "Not applicable"),
)



# Generated by Django 5.0 on 2024-01-25 05:33

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0137_alter_bloodresultsfbc_platelets_value_and_more"),
        ("sites", "0002_alter_domain_unique"),
    ]

    operations = [
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsglu",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultshba1c",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsins",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslipid",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsrft",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="cd4result",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="clinicalnote",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="clinicalreview",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="clinicalreviewbaseline",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="complicationsbaseline",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="complicationsfollowup",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="concomitantmedication",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="dminitialreview",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="dmmedicationadherence",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="dmreview",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="drugrefilldm",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="drugrefillhiv",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="drugrefillhtn",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="eq5d3l",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="familyhistory",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="glucose",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomics",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsassets",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicspatient",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsproperty",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsglu",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultshba1c",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsins",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslipid",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsrft",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcd4result",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalclinicalnote",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalclinicalreview",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalclinicalreviewbaseline",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcomplicationsbaseline",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcomplicationsfollowup",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalconcomitantmedication",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldminitialreview",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldmmedicationadherence",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldmreview",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldrugrefilldm",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldrugrefillhiv",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldrugrefillhtn",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicaleq5d3l",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalfamilyhistory",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalglucose",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomics",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsassets",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsincome",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicspatient",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsproperty",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhivinitialreview",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhivmedicationadherence",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhivreview",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhtninitialreview",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhtnmedicationadherence",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhtnreview",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalicecapa",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicallocationupdate",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmalariatest",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmedications",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalnextappointment",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalotherbaselinedata",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsocialharms",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectrequisition",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectvisit",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectvisitmissed",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalurinedipsticktest",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalurinepregnancy",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalviralloadresult",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalvitals",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="hivinitialreview",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="hivmedicationadherence",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="hivreview",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="htninitialreview",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="htnmedicationadherence",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="htnreview",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="icecapa",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="locationupdate",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="malariatest",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="medications",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="nextappointment",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="otherbaselinedata",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="socialharms",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="subjectrequisition",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="subjectvisit",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="subjectvisitmissed",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="urinedipsticktest",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="urinepregnancy",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="viralloadresult",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="vitals",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
    ]



from edc_crf.model_mixins import CrfStatusModelMixin
from edc_lab.model_mixins import RequisitionModelMixin
from edc_model.models import BaseUuidModel


class SubjectRequisition(RequisitionModelMixin, CrfStatusModelMixin, BaseUuidModel):
    class Meta(RequisitionModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Subject requisition"
        verbose_name_plural = "Subject requisitions"
        indexes = RequisitionModelMixin.Meta.indexes + BaseUuidModel.Meta.indexes



from django import forms
from edc_action_item.forms import ActionItemCrfFormMixin
from edc_crf.crf_form_validator import CrfFormValidator
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_lab_panel.panels import lipids_panel
from edc_lab_results.form_validator_mixins import BloodResultsFormValidatorMixin

from ...models import BloodResultsLipid


class BloodResultsLipidFormValidator(BloodResultsFormValidatorMixin, CrfFormValidator):
    panel = lipids_panel


class BloodResultsLipidForm(ActionItemCrfFormMixin, CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = BloodResultsLipidFormValidator

    class Meta(ActionItemCrfFormMixin.Meta):
        model = BloodResultsLipid
        fields = "__all__"



from .patient_log import PatientLog


class PatientLogUg(PatientLog):
    class Meta:
        proxy = True
        verbose_name = "Patient Log (Uganda)"
        verbose_name_plural = "Patient Log (Uganda)"



# Generated by Django 5.0.4 on 2024-04-15 20:48

import intecomm_subject.models.fields.duration_field
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_subject", "0170_remove_careseekinga_accompany_lost_income_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="careseekinga",
            name="travel_time",
            field=intecomm_subject.models.fields.duration_field.DurationField(
                metadata="FTRATIME1",
                null=True,
                verbose_name="How long did it take you to reach here?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekinga",
            name="travel_time",
            field=intecomm_subject.models.fields.duration_field.DurationField(
                metadata="FTRATIME1",
                null=True,
                verbose_name="How long did it take you to reach here?",
            ),
        ),
    ]



from django.contrib import admin
from edc_action_item import action_fieldset_tuple
from edc_lab_panel.panels import blood_glucose_panel
from edc_lab_results.admin import BloodResultsModelAdminMixin
from edc_lab_results.fieldsets import BloodResultFieldset

from ...admin_site import intecomm_subject_admin
from ...forms import BloodResultsGluForm
from ...models import BloodResultsGlu
from ..modeladmin_mixins import CrfModelAdmin


@admin.register(BloodResultsGlu, site=intecomm_subject_admin)
class BloodResultsGluAdmin(BloodResultsModelAdminMixin, CrfModelAdmin):
    form = BloodResultsGluForm
    fieldsets = BloodResultFieldset(
        blood_glucose_panel,
        extra_fieldsets=[
            (2, ("Fasting", {"fields": ["fasting"]})),
            (-1, action_fieldset_tuple),
        ],
    ).fieldsets

    radio_fields = {
        "fasting": admin.VERTICAL,
        "results_abnormal": admin.VERTICAL,
        "results_reportable": admin.VERTICAL,
    }



# Generated by Django 4.2.1 on 2023-05-24 01:42

from django.db import migrations, models
import edc_model_fields.fields.other_charfield


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_subject",
            "0045_healtheconomicshouseholdhead_hoh_marital_status_other_and_more",
        ),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="healtheconomicsassets",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Health Economics: Assets",
                "verbose_name_plural": "Health Economics: Assets",
            },
        ),
        migrations.AlterModelOptions(
            name="healtheconomicshouseholdhead",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Health Economics: Household head",
                "verbose_name_plural": "Health Economics: Household head",
            },
        ),
        migrations.AlterModelOptions(
            name="healtheconomicsincome",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Health Economics: Income",
                "verbose_name_plural": "Health Economics: Income",
            },
        ),
        migrations.AlterModelOptions(
            name="healtheconomicspatient",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Health Economics: Patient",
                "verbose_name_plural": "Health Economics: Patient",
            },
        ),
        migrations.AlterModelOptions(
            name="healtheconomicsproperty",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Health Economics: Property",
                "verbose_name_plural": "Health Economics: Property",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalhealtheconomicsassets",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Health Economics: Assets",
                "verbose_name_plural": "historical Health Economics: Assets",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalhealtheconomicshouseholdhead",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Health Economics: Household head",
                "verbose_name_plural": "historical Health Economics: Household head",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalhealtheconomicsincome",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Health Economics: Income",
                "verbose_name_plural": "historical Health Economics: Income",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalhealtheconomicspatient",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Health Economics: Patient",
                "verbose_name_plural": "historical Health Economics: Patient",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalhealtheconomicsproperty",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Health Economics: Property",
                "verbose_name_plural": "historical Health Economics: Property",
            },
        ),
        migrations.AddField(
            model_name="healtheconomicspatient",
            name="pat_education_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER level of education, specify ...",
            ),
        ),
        migrations.AddField(
            model_name="healtheconomicspatient",
            name="pat_ethnicity_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER ethnic background, specify ...",
            ),
        ),
        migrations.AddField(
            model_name="healtheconomicspatient",
            name="pat_insurance_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER health insurance status, specify ...",
            ),
        ),
        migrations.AddField(
            model_name="healtheconomicspatient",
            name="pat_marital_status_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER marital status, specify ...",
            ),
        ),
        migrations.AddField(
            model_name="healtheconomicspatient",
            name="pat_religion_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER religious orientation, specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicalhealtheconomicspatient",
            name="pat_education_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER level of education, specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicalhealtheconomicspatient",
            name="pat_ethnicity_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER ethnic background, specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicalhealtheconomicspatient",
            name="pat_insurance_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER health insurance status, specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicalhealtheconomicspatient",
            name="pat_marital_status_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER marital status, specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicalhealtheconomicspatient",
            name="pat_religion_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If OTHER religious orientation, specify ...",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsassets",
            name="residence_ownership",
            field=models.CharField(
                choices=[
                    ("renter", "Rent"),
                    ("owner", "Own themselves"),
                    ("family_owned", "Owned by someone else in family"),
                    ("nonfamily_owned", "Owned by someone else other than family member"),
                    ("joint_owned", "Owned together with someone"),
                ],
                max_length=25,
                verbose_name="Is the house you live in rented, owned by you (either on your own, or with someone else), or owned by someone else in your family?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_employment",
            field=models.CharField(
                choices=[
                    ("1", "Full time employed"),
                    ("2", "Regular part time employed "),
                    ("3", "Irregular/ occasional/ day worker employment"),
                    ("4", "Non-paid/ voluntary role "),
                    ("5", "Student"),
                    ("6", "Homemaker"),
                    ("7", "Unemployed (able to work)"),
                    ("8", "Unemployed (unable to work)"),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applcable"),
                ],
                max_length=25,
                verbose_name="Household head’s employment status",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_employment_type",
            field=models.CharField(
                choices=[
                    ("1", "Legislators, administrators, and managers"),
                    ("2", "Professionals"),
                    ("3", "Technicians and associate professionals"),
                    ("4", "Clerks"),
                    ("5", "Service workers and shop sale workers"),
                    ("6", "Skilled agricultural and fishery workers"),
                    ("7", "Unskilled agricultural and fishery workers"),
                    ("8", "Craft and related workers"),
                    ("9", "Plant and machine operators and assemblers"),
                    ("10", "Elementary occupations"),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="Household head’s type of employment",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_marital_status",
            field=models.CharField(
                choices=[
                    ("1", "Never Married (but not co-habiting)"),
                    ("2", "Co-habiting"),
                    ("3", "Currently Married"),
                    ("4", "Separated/Divorced"),
                    ("5", "Widowed"),
                    ("OTHER", "Other, specify ..."),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="Household head’s marital status",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicspatient",
            name="pat_employment",
            field=models.CharField(
                choices=[
                    ("1", "Full time employed"),
                    ("2", "Regular part time employed "),
                    ("3", "Irregular/ occasional/ day worker employment"),
                    ("4", "Non-paid/ voluntary role "),
                    ("5", "Student"),
                    ("6", "Homemaker"),
                    ("7", "Unemployed (able to work)"),
                    ("8", "Unemployed (unable to work)"),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applcable"),
                ],
                max_length=25,
                verbose_name="What is your employment status?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicspatient",
            name="pat_employment_type",
            field=models.CharField(
                choices=[
                    ("1", "Legislators, administrators, and managers"),
                    ("2", "Professionals"),
                    ("3", "Technicians and associate professionals"),
                    ("4", "Clerks"),
                    ("5", "Service workers and shop sale workers"),
                    ("6", "Skilled agricultural and fishery workers"),
                    ("7", "Unskilled agricultural and fishery workers"),
                    ("8", "Craft and related workers"),
                    ("9", "Plant and machine operators and assemblers"),
                    ("10", "Elementary occupations"),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="What is your type of employment?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicspatient",
            name="pat_marital_status",
            field=models.CharField(
                choices=[
                    ("1", "Never Married (but not co-habiting)"),
                    ("2", "Co-habiting"),
                    ("3", "Currently Married"),
                    ("4", "Separated/Divorced"),
                    ("5", "Widowed"),
                    ("OTHER", "Other, specify ..."),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="What is your marital status?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsassets",
            name="residence_ownership",
            field=models.CharField(
                choices=[
                    ("renter", "Rent"),
                    ("owner", "Own themselves"),
                    ("family_owned", "Owned by someone else in family"),
                    ("nonfamily_owned", "Owned by someone else other than family member"),
                    ("joint_owned", "Owned together with someone"),
                ],
                max_length=25,
                verbose_name="Is the house you live in rented, owned by you (either on your own, or with someone else), or owned by someone else in your family?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_employment",
            field=models.CharField(
                choices=[
                    ("1", "Full time employed"),
                    ("2", "Regular part time employed "),
                    ("3", "Irregular/ occasional/ day worker employment"),
                    ("4", "Non-paid/ voluntary role "),
                    ("5", "Student"),
                    ("6", "Homemaker"),
                    ("7", "Unemployed (able to work)"),
                    ("8", "Unemployed (unable to work)"),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applcable"),
                ],
                max_length=25,
                verbose_name="Household head’s employment status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_employment_type",
            field=models.CharField(
                choices=[
                    ("1", "Legislators, administrators, and managers"),
                    ("2", "Professionals"),
                    ("3", "Technicians and associate professionals"),
                    ("4", "Clerks"),
                    ("5", "Service workers and shop sale workers"),
                    ("6", "Skilled agricultural and fishery workers"),
                    ("7", "Unskilled agricultural and fishery workers"),
                    ("8", "Craft and related workers"),
                    ("9", "Plant and machine operators and assemblers"),
                    ("10", "Elementary occupations"),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="Household head’s type of employment",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_marital_status",
            field=models.CharField(
                choices=[
                    ("1", "Never Married (but not co-habiting)"),
                    ("2", "Co-habiting"),
                    ("3", "Currently Married"),
                    ("4", "Separated/Divorced"),
                    ("5", "Widowed"),
                    ("OTHER", "Other, specify ..."),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="Household head’s marital status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicspatient",
            name="pat_employment",
            field=models.CharField(
                choices=[
                    ("1", "Full time employed"),
                    ("2", "Regular part time employed "),
                    ("3", "Irregular/ occasional/ day worker employment"),
                    ("4", "Non-paid/ voluntary role "),
                    ("5", "Student"),
                    ("6", "Homemaker"),
                    ("7", "Unemployed (able to work)"),
                    ("8", "Unemployed (unable to work)"),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applcable"),
                ],
                max_length=25,
                verbose_name="What is your employment status?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicspatient",
            name="pat_employment_type",
            field=models.CharField(
                choices=[
                    ("1", "Legislators, administrators, and managers"),
                    ("2", "Professionals"),
                    ("3", "Technicians and associate professionals"),
                    ("4", "Clerks"),
                    ("5", "Service workers and shop sale workers"),
                    ("6", "Skilled agricultural and fishery workers"),
                    ("7", "Unskilled agricultural and fishery workers"),
                    ("8", "Craft and related workers"),
                    ("9", "Plant and machine operators and assemblers"),
                    ("10", "Elementary occupations"),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="What is your type of employment?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicspatient",
            name="pat_marital_status",
            field=models.CharField(
                choices=[
                    ("1", "Never Married (but not co-habiting)"),
                    ("2", "Co-habiting"),
                    ("3", "Currently Married"),
                    ("4", "Separated/Divorced"),
                    ("5", "Widowed"),
                    ("OTHER", "Other, specify ..."),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="What is your marital status?",
            ),
        ),
    ]



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from edc_model.models import BaseUuidModel
from edc_rx.model_mixins import DrugRefillModelMixin

from ..model_mixins import CrfModelMixin
from .arv_regimens import ArvRegimens


class DrugRefillHiv(DrugRefillModelMixin, CrfModelMixin, BaseUuidModel):
    rx = models.ForeignKey(
        ArvRegimens,
        verbose_name="Which medicine did the patient receive today?",
        on_delete=models.PROTECT,
    )

    clinic_days = models.IntegerField(
        verbose_name="How many days supplied by the clinic",
        validators=[MinValueValidator(0), MaxValueValidator(180)],
        help_text="days",
    )

    club_days = models.IntegerField(
        verbose_name="How many days supplied by a club",
        validators=[MinValueValidator(0), MaxValueValidator(180)],
        help_text="days",
    )

    purchased_days = models.IntegerField(
        verbose_name="How many days supplied by to be purchased",
        validators=[MinValueValidator(0), MaxValueValidator(180)],
        help_text=(
            "This can be purchased by patient, through a medicines club "
            "that the patient belong to, through insurance or someone else has paid. "
        ),
    )

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Drug Refill: HIV"
        verbose_name_plural = "Drug Refills: HIV"
        indexes = CrfModelMixin.Meta.indexes + BaseUuidModel.Meta.indexes



from django.db import models
from edc_constants.choices import YES_NO_NA
from edc_constants.constants import NOT_APPLICABLE
from edc_dx_review.model_mixins import rx_initial_review_model_mixin_factory
from edc_model.models import BaseUuidModel

from ..model_mixins import (
    CrfModelMixin,
    FollowupReviewModelMixin,
    ViralLoadResultModelMixin,
)


# TODO: Ask for viral load if available since last visit
class HivReview(
    rx_initial_review_model_mixin_factory(
        "rx_init", verbose_name_label="antiretroviral therapy (ART)"
    ),
    ViralLoadResultModelMixin,
    FollowupReviewModelMixin,
    CrfModelMixin,
    BaseUuidModel,
):
    dx = models.CharField(
        verbose_name="Has the patient been infected with HIV?",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    @property
    def best_art_initiation_date(self):
        return self.rx_init_date or self.rx_init_calculated_date

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "HIV Review"
        verbose_name_plural = "HIV Review"
        indexes = CrfModelMixin.Meta.indexes + BaseUuidModel.Meta.indexes



# Generated by Django 4.2.7 on 2023-12-04 22:19

from django.db import migrations, models
import edc_utils.date


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_group", "0016_alter_communitycarelocation_device_created_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="communitycarelocation",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Community Care Location",
                "verbose_name_plural": "Community Care Locations",
            },
        ),
        migrations.AlterModelOptions(
            name="patientfollowupcall",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Patient Followup Call",
                "verbose_name_plural": "Patient Followup Calls",
            },
        ),
        migrations.AlterModelOptions(
            name="patientgroup",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Patient Group",
                "verbose_name_plural": "Patient Groups",
            },
        ),
        migrations.AlterModelOptions(
            name="patientgroupappointment",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Patient Group Appointment",
                "verbose_name_plural": "Patient Group Appointments",
            },
        ),
        migrations.AlterModelOptions(
            name="patientgroupmeeting",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Patient Group Meeting",
                "verbose_name_plural": "Patient Groups Meeting",
            },
        ),
        migrations.AddField(
            model_name="communitycarelocation",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="communitycarelocation",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalpatientfollowupcall",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalpatientfollowupcall",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalpatientgroup",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalpatientgroup",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="patientfollowupcall",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="patientfollowupcall",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="patientgroup",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="patientgroup",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="patientgroupappointment",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="patientgroupappointment",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="patientgroupmeeting",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="patientgroupmeeting",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalpatientfollowupcall",
            name="report_datetime",
            field=models.DateTimeField(db_index=True, default=edc_utils.date.get_utcnow),
        ),
        migrations.AlterField(
            model_name="patientfollowupcall",
            name="report_datetime",
            field=models.DateTimeField(db_index=True, default=edc_utils.date.get_utcnow),
        ),
        migrations.AddIndex(
            model_name="communitycarelocation",
            index=models.Index(
                fields=["modified", "created"], name="intecomm_gr_modifie_9695bc_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="communitycarelocation",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="intecomm_gr_user_mo_b4b5cb_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="patientfollowupcall",
            index=models.Index(
                fields=["modified", "created"], name="intecomm_gr_modifie_55b38c_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="patientfollowupcall",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="intecomm_gr_user_mo_b1771e_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="patientgroup",
            index=models.Index(
                fields=["modified", "created"], name="intecomm_gr_modifie_709aa8_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="patientgroup",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="intecomm_gr_user_mo_88354b_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="patientgroup",
            index=models.Index(
                fields=["group_identifier", "status"],
                name="intecomm_gr_group_i_d392d3_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="patientgroupappointment",
            index=models.Index(
                fields=["modified", "created"], name="intecomm_gr_modifie_bc2cdb_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="patientgroupappointment",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="intecomm_gr_user_mo_aaafcb_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="patientgroupmeeting",
            index=models.Index(
                fields=["modified", "created"], name="intecomm_gr_modifie_12dbed_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="patientgroupmeeting",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="intecomm_gr_user_mo_746ccc_idx",
            ),
        ),
    ]



# Generated by Django 5.0.4 on 2024-04-12 02:01

import edc_model_fields.fields.custom_django_fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_subject", "0166_careseekinga_med_collected_location_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="careseekinga",
            name="accompany_num",
            field=edc_model_fields.fields.custom_django_fields.IntegerField2(
                blank=True,
                metadata="FACMP1",
                null=True,
                verbose_name="Number of people who accompanied you here today",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekinga",
            name="accompany_num",
            field=edc_model_fields.fields.custom_django_fields.IntegerField2(
                blank=True,
                metadata="FACMP1",
                null=True,
                verbose_name="Number of people who accompanied you here today",
            ),
        ),
    ]



from django.db import models
from edc_sites.managers import CurrentSiteManager as BaseCurrentSiteManager

from .ae_initial import AeInitial


class BaseManager(models.Manager):
    use_in_migrations = True

    def get_by_natural_key(self, action_identifier, ae_initial_action_identifier):
        ae_initial = AeInitial.objects.get(action_identifier=ae_initial_action_identifier)
        return self.get(action_identifier=action_identifier, ae_initial=ae_initial)


class AeManager(BaseManager):
    use_in_migrations = True


class CurrentSiteManager(BaseManager, BaseCurrentSiteManager):
    use_in_migrations = True



# Generated by Django 4.2.4 on 2023-08-26 22:51

import django.contrib.sites.managers
from django.db import migrations

import intecomm_consent.models.subject_consent


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_consent", "0012_alter_historicalsubjectconsent_device_created_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="SubjectConsentUg",
            fields=[],
            options={
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("intecomm_consent.subjectconsent",),
            managers=[
                ("on_site", django.contrib.sites.managers.CurrentSiteManager()),
                ("objects", intecomm_consent.models.subject_consent.ConsentObjectsManager()),
            ],
        ),
    ]



from django.utils.translation import gettext as _
from edc_model.validators import hm_validator
from edc_model_fields.fields import CharField2


class DurationField(CharField2):
    def __init__(self, *args, **kwargs):
        self.custom_null = kwargs.get("null")
        self.custom_blank = kwargs.get("blank")
        if self.custom_blank is None:
            kwargs.update(blank=True)
        if self.custom_null is None:
            kwargs.update(null=True)
        kwargs.update(max_length=15)
        kwargs.update(validators=[hm_validator])
        kwargs.update(
            help_text=_(
                "Please insert a numeric value followed by “h” for hours, "
                "and a numeric value followed by “m” for minutes. For example, 1h2m, 0h35m, "
                "and so on"
            )
        )
        super().__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        if self.custom_null is None:
            del kwargs["null"]
        if self.custom_blank is None:
            del kwargs["blank"]
        del kwargs["max_length"]
        del kwargs["validators"]
        del kwargs["help_text"]
        return name, path, args, kwargs



from django.db import models
from django_crypto_fields.fields import EncryptedCharField, EncryptedTextField
from edc_constants.choices import YES_NO, YES_NO_UNSURE_NA
from edc_constants.constants import NOT_APPLICABLE
from edc_model.models import HistoricalRecords, OtherCharField
from edc_model.validators.phone import phone_number
from edc_sites.managers import CurrentSiteManager
from edc_utils import get_utcnow

from ..choices import RESPONDENT_CHOICES


class Manager(models.Manager):
    """A manager class for Crf models, models that have an FK to
    the visit model.
    """

    use_in_migrations = True

    def get_by_natural_key(self, patient_log):
        return self.get(patient_log=patient_log)


class PatientCallModelMixin(models.Model):
    report_datetime = models.DateTimeField(default=get_utcnow, db_index=True)

    contact_number = EncryptedCharField(blank=False, validators=[phone_number])

    alt_contact_number = EncryptedCharField(null=True, blank=True, validators=[phone_number])

    answered = models.CharField(max_length=15, choices=YES_NO, null=True, blank=False)

    respondent = models.CharField(
        max_length=15, choices=RESPONDENT_CHOICES, default=NOT_APPLICABLE
    )

    survival_status = models.CharField(
        verbose_name="Is the participant known to be alive",
        max_length=15,
        choices=YES_NO_UNSURE_NA,
        default=NOT_APPLICABLE,
    )

    catchment_area = models.CharField(
        verbose_name="Does the participant still reside within the catchment area",
        max_length=15,
        choices=YES_NO_UNSURE_NA,
        default=NOT_APPLICABLE,
    )

    last_appt_date = models.DateField(
        verbose_name="When did the patient last seek care",
        null=True,
        blank=True,
        help_text="This may be helpful when updating health records",
    )

    last_attend_clinic = models.CharField(
        verbose_name="Where did the patient last seek care",
        max_length=25,
        # choices=(),
        null=True,
        blank=True,
        help_text="This may be helpful when updating health records",
    )

    last_attend_clinic_other = OtherCharField()

    next_appt_date = models.DateField(
        verbose_name="When will the patient next attend", null=True, blank=True
    )

    call_again = models.CharField(
        verbose_name="May we call again?", max_length=15, choices=YES_NO
    )

    comment = EncryptedTextField(verbose_name="Note", null=True, blank=True)

    objects = Manager()
    on_site = CurrentSiteManager()
    history = HistoricalRecords(inherit=True)

    class Meta:
        abstract = True



from edc_consent.managers import ConsentObjectsByCdefManager, CurrentSiteByCdefManager

from .subject_consent import SubjectConsent


class SubjectConsentUg(SubjectConsent):

    objects = ConsentObjectsByCdefManager()

    on_site = CurrentSiteByCdefManager()

    class Meta(SubjectConsent.Meta):
        proxy = True
        verbose_name = "Subject Consent (Uganda)"
        verbose_name_plural = "Subject Consents (Uganda)"



from __future__ import annotations

from django.core.validators import MaxValueValidator, MinValueValidator
from django.utils.translation import gettext as _
from edc_constants.choices import YES_NO, YES_NO_NA
from edc_constants.constants import NOT_APPLICABLE
from edc_model.models import BaseUuidModel
from edc_model_fields.fields import (
    CharField2,
    IntegerField2,
    ManyToManyField2,
    OtherCharField,
)

from intecomm_lists.models import Conditions, MoneySources, TravelMethods, VisitReasons

from ...choices import (
    ACCOMPANIED_BY,
    FACILITY_VISIT_ALTERNATIVES,
    FACILITY_VISIT_ALTERNATIVES_NA,
    MED_COLLECTION_LOCATIONS,
    MONEY_SOURCES,
    NOT_COLLECTED_REASONS,
    REFERRAL_FACILITY,
    REFERRAL_TYPE,
    TESTS_NOT_DONE_REASONS,
)
from ...model_mixins import CrfModelMixin
from ..fields import DurationField, ExpenseField


def convert_to_choices(s: str) -> tuple:
    choices = []
    x = s.split("\n")
    for item in x:
        values = item.split("\t")
        choices.append(tuple(reversed(values)))
    return tuple(choices)


class CareseekingA(CrfModelMixin, BaseUuidModel):

    travel_method = ManyToManyField2(
        TravelMethods,
        verbose_name=_("How did you travel here?"),
        metadata="FTRA1",
    )

    travel_time = DurationField(
        verbose_name="How long did it take you to reach here?",
        null=True,
        blank=False,
        metadata="FTRATIME1",
    )

    travel_cost = ExpenseField(
        verbose_name=_(
            "Thinking about yourself and anyone that accompanied you, "
            "how much was spent on travel from your home to reach here?"
        ),
        null=True,
        blank=False,
        metadata="FTRAVCOST1",
    )

    food_cost = ExpenseField(
        verbose_name=_(
            "Thinking about yourself and anyone that accompanied you, how much did you "
            "have to pay for food, drink or other refreshments during your travel or "
            "during your visit? (e.g. food, drink, etc.)"
        ),
        null=True,
        blank=False,
        metadata="FFOODCOST1",
    )

    care_visit_reason = ManyToManyField2(
        VisitReasons,
        verbose_name=_("What was the reason for today’s visit?"),
        metadata="FMEDCOND1",
    )

    care_visit_reason_other = OtherCharField(metadata="FMEDCONDOTHER1")

    care_visit_cost = ExpenseField(
        verbose_name=_(
            "How much money did you spend on healthworker and consultation "
            "fees during today’s visit?"
        ),
        null=False,
        blank=False,
        metadata="FFEECOST1",
    )

    med_prescribed = CharField2(
        verbose_name=_("Were you prescribed any medicines during today’s visit?"),
        max_length=25,
        choices=YES_NO,
        metadata="FMED1",
    )

    med_conditions = ManyToManyField2(
        Conditions,
        verbose_name=_("What were the medicines were for?"),
        blank=True,
        metadata="FMEDCOND1",
    )

    med_conditions_other = OtherCharField(metadata="FMEDCONDOTHER1")

    med_collected = CharField2(
        verbose_name=_(
            "Did you receive/collect these medicines (whether paid or received for free)?"
        ),
        max_length=25,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        metadata="FMEDCOLL1",
    )

    med_not_collected_reason = CharField2(
        verbose_name=_("Why were these medicines not received/collected?"),
        max_length=25,
        choices=NOT_COLLECTED_REASONS(),
        default=NOT_APPLICABLE,
        metadata="FMED1",
    )

    med_not_collected_reason_other = OtherCharField(metadata="FMEDOTHER1")

    med_cost_tot = ExpenseField(
        verbose_name=_("How much was spent on these medicines? "),
        metadata="FMEDCOST1",
    )

    med_cost_hiv = ExpenseField(
        verbose_name=_("How much was spent on HIV medicines? "),
        metadata="FMEDCOSTHIV1",
    )

    med_cost_htn = ExpenseField(
        verbose_name=_("How much was spent on Hypertension medicines? "),
        metadata="FMEDCOSTHTN1",
    )

    med_cost_dm = ExpenseField(
        verbose_name=_("How much was spent on Diabetes medicines? "),
        metadata="FMEDCOSTDM1",
    )

    med_cost_other = ExpenseField(
        verbose_name=_("How much was spent on OTHER medicines? "),
        metadata="FMEDCOSTOTHER1",
    )

    med_collected_location = CharField2(
        verbose_name="Where did you collect the medicines from?",
        max_length=25,
        choices=MED_COLLECTION_LOCATIONS(),
        default=NOT_APPLICABLE,
        metadata="FMEDCOLLECTLOC1",
    )

    med_collected_location_other = OtherCharField(
        metadata="FMEDCOLLECTLOCOTHER1",
    )

    tests_requested = CharField2(
        verbose_name=_(
            "Did the healthcare worker request for any tests to be done during this visit?"
        ),
        max_length=25,
        choices=YES_NO,
        metadata="FTEST1",
    )

    tests_done = CharField2(
        verbose_name=_("Were the tests performed?"),
        max_length=25,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        metadata="FTESTDONE1",
    )

    tests_not_done_reason = CharField2(
        verbose_name=_("Why were the tests not performed?"),
        max_length=25,
        choices=TESTS_NOT_DONE_REASONS(),
        default=NOT_APPLICABLE,
        metadata="FTESTNO1",
    )

    tests_not_done_other = OtherCharField(metadata="FTESTNOOTHER1")

    tests_cost = ExpenseField(
        verbose_name=_("How much did you spend on these tests?"),
        metadata="FTESTCOST1",
    )

    care_visit_duration = DurationField(
        verbose_name=_(
            "How much time did you spend during your visit today -- "
            "from arrival to this place until the end of your visit?"
        ),
        metadata="FFACTIME1",
    )

    with_hcw_duration = DurationField(
        verbose_name=_(
            "How much time did you spend during the consultation "
            "(i.e. time spent with the healthcareworker)?"
        ),
        metadata="FWORKTIME1",
    )

    missed_activities = CharField2(
        verbose_name=_(
            "If you were not attending the visit today, what would you have been doing?"
        ),
        max_length=25,
        choices=FACILITY_VISIT_ALTERNATIVES(),
        metadata="FACTIVITY1",
    )

    missed_activities_other = OtherCharField(metadata="FMEDOTHER1")

    referral = CharField2(
        verbose_name=_(
            "As a result of your visit today, have you been referred for further care?"
        ),
        max_length=15,
        choices=YES_NO,
        metadata="FREFER1",
    )

    referral_type = CharField2(
        verbose_name=_("Is this for inpatient or outpatient care?"),
        max_length=25,
        choices=REFERRAL_TYPE(),
        default=NOT_APPLICABLE,
        metadata="FREFTYPE1",
    )

    referral_facility = CharField2(
        verbose_name=_("What type of facility have you been referred to?"),
        max_length=25,
        choices=REFERRAL_FACILITY(),
        default=NOT_APPLICABLE,
        metadata="FREFAC1",
    )

    accompany = CharField2(
        verbose_name="Who accompanied you here today?",
        max_length=25,
        null=True,
        blank=False,
        choices=ACCOMPANIED_BY(),
        metadata="FACMP1",
    )

    accompany_num = IntegerField2(
        verbose_name=_("Number of people who accompanied you here today"),
        null=True,
        blank=True,
        validators=[MinValueValidator(1), MaxValueValidator(15)],
        metadata="FACMP1",
    )

    accompany_wait = CharField2(
        verbose_name="Did the people accompanying you wait for you during the visit?",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        metadata="FACMPWAIT1",
    )

    accompany_alt = CharField2(
        verbose_name=(
            "If those accompanying you were not attending the visit with you today, "
            "what would they have been doing?"
        ),
        max_length=25,
        choices=FACILITY_VISIT_ALTERNATIVES_NA(),
        default=NOT_APPLICABLE,
        metadata="FACMPACT1",
    )

    accompany_alt_other = OtherCharField(metadata="FACMPACTOTHER")

    money_sources = ManyToManyField2(
        MoneySources,
        verbose_name=_(
            "Thinking about the expenses you have reported for today’s visit, what were "
            "the source(s) of payment for all these expenses?"
        ),
        limit_choices_to={"name__in": [n[0] for n in MONEY_SOURCES if n[0] != NOT_APPLICABLE]},
        help_text="Select up to three sources. If 'other', please specify.",
        metadata="FTODSOURCE",
    )

    money_sources_other = OtherCharField(
        verbose_name=_("If other 'source of payment', please specify ..."),
        metadata="FTODSOURCEOTHER",
    )

    money_source_main = CharField2(
        verbose_name=_(
            "Of the various sources that you have just mentioned, "
            "what was the main source of payment?"
        ),
        max_length=25,
        choices=MONEY_SOURCES(),
        default=NOT_APPLICABLE,
        metadata="FTODSOURCEMAIN1",
    )

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Cost of Careseeking: Part A"
        verbose_name_plural = "Cost of Careseeking: Part A"
        indexes = CrfModelMixin.Meta.indexes + BaseUuidModel.Meta.indexes



# Generated by Django 4.2.7 on 2023-12-04 22:19

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_lists",
            "0009_alter_arvdrugs_options_alter_arvregimens_options_and_more",
        ),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name="arvdrugs",
            name="intecomm_li_name_b55bdc_idx",
        ),
        migrations.RemoveIndex(
            model_name="arvregimens",
            name="intecomm_li_name_dd4128_idx",
        ),
        migrations.RemoveIndex(
            model_name="clinicalwithdrawalreasons",
            name="intecomm_li_name_bfca7f_idx",
        ),
        migrations.RemoveIndex(
            model_name="clinicservices",
            name="intecomm_li_name_deec16_idx",
        ),
        migrations.RemoveIndex(
            model_name="conditions",
            name="intecomm_li_name_b9ae61_idx",
        ),
        migrations.RemoveIndex(
            model_name="consentrefusalreasons",
            name="intecomm_li_name_04461e_idx",
        ),
        migrations.RemoveIndex(
            model_name="dmmanagement",
            name="intecomm_li_name_edefb8_idx",
        ),
        migrations.RemoveIndex(
            model_name="dmtreatments",
            name="intecomm_li_name_596944_idx",
        ),
        migrations.RemoveIndex(
            model_name="drugdispensaries",
            name="intecomm_li_name_2312cd_idx",
        ),
        migrations.RemoveIndex(
            model_name="drugdispensers",
            name="intecomm_li_name_4d9885_idx",
        ),
        migrations.RemoveIndex(
            model_name="drugpaysources",
            name="intecomm_li_name_b764fd_idx",
        ),
        migrations.RemoveIndex(
            model_name="healthadvisors",
            name="intecomm_li_name_dd5fb2_idx",
        ),
        migrations.RemoveIndex(
            model_name="healthfacilitytypes",
            name="intecomm_li_name_af9934_idx",
        ),
        migrations.RemoveIndex(
            model_name="healthinterventiontypes",
            name="intecomm_li_name_5a8bd1_idx",
        ),
        migrations.RemoveIndex(
            model_name="healthservices",
            name="intecomm_li_name_b9dd17_idx",
        ),
        migrations.RemoveIndex(
            model_name="healthtalkconditions",
            name="intecomm_li_name_9d805e_idx",
        ),
        migrations.RemoveIndex(
            model_name="healthtalktypes",
            name="intecomm_li_name_dae47f_idx",
        ),
        migrations.RemoveIndex(
            model_name="htnmanagement",
            name="intecomm_li_name_17a894_idx",
        ),
        migrations.RemoveIndex(
            model_name="htntreatments",
            name="intecomm_li_name_ebbf62_idx",
        ),
        migrations.RemoveIndex(
            model_name="laboratorytests",
            name="intecomm_li_name_3dad94_idx",
        ),
        migrations.RemoveIndex(
            model_name="locationtypes",
            name="intecomm_li_name_f97934_idx",
        ),
        migrations.RemoveIndex(
            model_name="medicationshortagereasons",
            name="intecomm_li_name_ad571f_idx",
        ),
        migrations.RemoveIndex(
            model_name="nonadherencereasons",
            name="intecomm_li_name_a7a6a5_idx",
        ),
        migrations.RemoveIndex(
            model_name="offstudyreasons",
            name="intecomm_li_name_aabb4d_idx",
        ),
        migrations.RemoveIndex(
            model_name="refillconditions",
            name="intecomm_li_name_085d7f_idx",
        ),
        migrations.RemoveIndex(
            model_name="rxmodificationreasons",
            name="intecomm_li_name_c1233d_idx",
        ),
        migrations.RemoveIndex(
            model_name="rxmodifications",
            name="intecomm_li_name_3f4bd2_idx",
        ),
        migrations.RemoveIndex(
            model_name="screeningrefusalreasons",
            name="intecomm_li_name_1da54a_idx",
        ),
        migrations.RemoveIndex(
            model_name="subjectvisitmissedreasons",
            name="intecomm_li_name_12d94d_idx",
        ),
        migrations.RemoveIndex(
            model_name="toxicitywithdrawalreasons",
            name="intecomm_li_name_30f17d_idx",
        ),
        migrations.RemoveIndex(
            model_name="transferreasons",
            name="intecomm_li_name_eb67c1_idx",
        ),
        migrations.RemoveIndex(
            model_name="transportchoices",
            name="intecomm_li_name_bea030_idx",
        ),
        migrations.RemoveIndex(
            model_name="visitreasons",
            name="intecomm_li_name_59f9c6_idx",
        ),
    ]



# Generated by Django 4.1.2 on 2022-11-21 18:17

import django.contrib.sites.models
from django.db import migrations, models
import django.db.models.deletion
import django_crypto_fields.fields.encrypted_char_field


class Migration(migrations.Migration):
    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        (
            "intecomm_screening",
            "0002_remove_healthtalklog_unique_lower_name_report_date_and_more",
        ),
    ]

    operations = [
        migrations.CreateModel(
            name="Site",
            fields=[],
            options={
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("sites.site",),
            managers=[
                ("objects", django.contrib.sites.models.SiteManager()),
            ],
        ),
        migrations.RemoveField(
            model_name="historicalpatientlog",
            name="patient_group",
        ),
        migrations.RemoveField(
            model_name="patientlog",
            name="patient_group",
        ),
        migrations.AlterField(
            model_name="historicalpatientlog",
            name="hf_identifier",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                db_index=True,
                help_text="Must be unique (Encryption: RSA local)",
                max_length=71,
                verbose_name="Health center identifier",
            ),
        ),
        migrations.AlterField(
            model_name="patientlog",
            name="hf_identifier",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text="Must be unique (Encryption: RSA local)",
                max_length=71,
                unique=True,
                verbose_name="Health center identifier",
            ),
        ),
        migrations.AlterField(
            model_name="historicalpatientlog",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="intecomm_screening.site",
                verbose_name="Health center",
            ),
        ),
        migrations.AlterField(
            model_name="patientlog",
            name="site",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="intecomm_screening.site",
                verbose_name="Health center",
            ),
        ),
    ]



from .appointment_form import AppointmentForm
from .blood_results import (
    BloodResultsFbcForm,
    BloodResultsGluForm,
    BloodResultsHba1cForm,
    BloodResultsInsForm,
    BloodResultsLftForm,
    BloodResultsLipidForm,
    BloodResultsRftForm,
)
from .cd4_result_form import Cd4ResultForm
from .clinical_note_form import ClinicalNoteForm
from .clinical_review_baseline_form import ClinicalReviewBaselineForm
from .clinical_review_form import ClinicalReviewForm
from .complications_baseline_form import ComplicationsBaselineForm
from .complications_followup_form import ComplicationsFollowupForm
from .concomitant_medication_form import ConcomitantMedicationForm
from .dm_initial_review_form import DmInitialReviewForm
from .dm_medication_adherence_form import DmMedicationAdherenceForm
from .dm_review_form import DmReviewForm
from .drug_refill_dm_form import DrugRefillDmForm
from .drug_refill_hiv_form import DrugRefillHivForm
from .drug_refill_htn_form import DrugRefillHtnForm
from .drug_supply_dm_form import DrugSupplyDmForm
from .drug_supply_hiv_form import DrugSupplyHivForm
from .drug_supply_htn_form import DrugSupplyHtnForm
from .eq5d3l_form import Eq5d3lForm
from .family_history_form import FamilyHistoryForm
from .health_economics import (
    CareseekingAForm,
    CareseekingBForm,
    HealthEconomicsAssetsForm,
    HealthEconomicsHouseholdHeadForm,
    HealthEconomicsIncomeForm,
    HealthEconomicsPatientForm,
    HealthEconomicsPropertyForm,
)
from .hiv_initial_review_form import HivInitialReviewForm
from .hiv_medication_adherence_form import HivMedicationAdherenceForm
from .hiv_review_form import HivReviewForm
from .htn_initial_review_form import HtnInitialReviewForm
from .htn_medication_adherence_form import HtnMedicationAdherenceForm
from .htn_review_form import HtnReviewForm
from .icecapa_form import IcecapaForm
from .location_update_form import LocationUpdateForm
from .malaria_test_form import MalariaTestForm
from .medications_form import MedicationsForm
from .next_appointment_form import NextAppointmentForm
from .other_baseline_data import OtherBaselineDataForm
from .social_harms_form import SocialHarmsForm
from .subject_requisition_form import SubjectRequisitionForm
from .subject_visit_form import SubjectVisitForm
from .subject_visit_missed_form import SubjectVisitMissedForm
from .urine_dipstick_test_form import UrineDipstickTestForm
from .urine_pregnancy_form import UrinePregnancyForm
from .vitals_form import VitalsForm



# Generated by Django 5.0.4 on 2024-04-10 21:36

import edc_model_fields.fields.other_charfield
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_subject", "0158_alter_careseekingb_med_collected_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="careseekingb",
            name="missed_activities_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                metadata="FMEDOTHER1",
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicalcareseekingb",
            name="missed_activities_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                metadata="FMEDOTHER1",
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
    ]



# Generated by Django 5.0.4 on 2024-04-17 03:36

import edc_model_fields.fields.custom_django_fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_subject", "0176_rename_seek_advice_careseekingb_accessed_care_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="careseekingb",
            name="accompany",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                choices=[
                    ("alone", "No one, I came alone"),
                    ("main_earner", "Main household earner"),
                    ("adult", "Other family member/relatives/friends (adults)"),
                    ("child", "Other family member/relatives/friends (children)"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                metadata="FOUTACMP1",
                null=True,
                verbose_name="Who accompanied you to your last/most recent visit?",
            ),
        ),
        migrations.AlterField(
            model_name="careseekingb",
            name="care_type",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                choices=[
                    ("outpatient", "Outpatient (includes laboratory testing)"),
                    ("inpatient", "Inpatient (stayed overnight for 1 or more nights)"),
                    ("both", "Both"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                metadata="FSEEKTYPE1",
                verbose_name="If yes, what type of care was this?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekingb",
            name="accompany",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                choices=[
                    ("alone", "No one, I came alone"),
                    ("main_earner", "Main household earner"),
                    ("adult", "Other family member/relatives/friends (adults)"),
                    ("child", "Other family member/relatives/friends (children)"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                metadata="FOUTACMP1",
                null=True,
                verbose_name="Who accompanied you to your last/most recent visit?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekingb",
            name="care_type",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                choices=[
                    ("outpatient", "Outpatient (includes laboratory testing)"),
                    ("inpatient", "Inpatient (stayed overnight for 1 or more nights)"),
                    ("both", "Both"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                metadata="FSEEKTYPE1",
                verbose_name="If yes, what type of care was this?",
            ),
        ),
    ]



# Generated by Django 5.0.4 on 2024-04-10 21:58

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_subject", "0160_careseekingb_inpatient_and_more"),
    ]

    operations = [
        migrations.RenameField(
            model_name="careseekingb",
            old_name="inpatient_accompany_nowork",
            new_name="inpatient_household_nowork",
        ),
        migrations.RenameField(
            model_name="careseekingb",
            old_name="inpatient_accompany_nowork_days",
            new_name="inpatient_household_nowork_days",
        ),
        migrations.RenameField(
            model_name="historicalcareseekingb",
            old_name="inpatient_accompany_nowork",
            new_name="inpatient_household_nowork",
        ),
        migrations.RenameField(
            model_name="historicalcareseekingb",
            old_name="inpatient_accompany_nowork_days",
            new_name="inpatient_household_nowork_days",
        ),
    ]



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from edc_constants.choices import YES_NO_PENDING_NA
from edc_constants.constants import NOT_APPLICABLE
from edc_lab.choices import VL_QUANTIFIER_NA
from edc_model.validators import date_not_future
from edc_reportable import COPIES_PER_MILLILITER


class ViralLoadResultModelMixin(models.Model):
    # Viral Load
    has_vl = models.CharField(
        verbose_name="Is the patient's most recent viral load result available?",
        max_length=25,
        choices=YES_NO_PENDING_NA,
        default=NOT_APPLICABLE,
        help_text="",
    )

    vl = models.IntegerField(
        verbose_name="Most recent viral load",
        validators=[MinValueValidator(0), MaxValueValidator(9999999)],
        null=True,
        blank=True,
        help_text=COPIES_PER_MILLILITER,
    )

    vl_quantifier = models.CharField(
        max_length=10,
        choices=VL_QUANTIFIER_NA,
        null=True,
        default=NOT_APPLICABLE,
    )

    drawn_date = models.DateField(
        verbose_name="Date specimen drawn",
        validators=[date_not_future],
        null=True,
        blank=True,
        help_text="If exact date not known, please estimate",
    )

    class Meta:
        abstract = True



from __future__ import annotations

from decimal import Decimal
from typing import Tuple

from django.core.exceptions import ObjectDoesNotExist
from django.urls import reverse
from edc_constants.constants import DM, HIV, HTN
from edc_randomization.site_randomizers import site_randomizers
from edc_randomization.utils import (
    SubjectNotRandomization,
    get_assignment_description_for_subject,
    get_assignment_for_subject,
)
from intecomm_rando.constants import COMMUNITY_ARM

from .exceptions import PatientGroupNotRandomized
from .models import PatientGroup
from .patient_group_updater import PatientGroupUpdater, PatientGroupUpdaterError


class PatientGroupRatioError(Exception):
    pass


def verify_patient_group_ratio_raise(
    patients, raise_on_outofrange=None
) -> Tuple[int, int, Decimal | None]:
    ncd = 0.0
    hiv = 0.0
    for patient_log in patients:
        if patient_log.conditions.filter(name__in=[DM, HTN]).exclude(name__in=[HIV]).exists():
            ncd += 1.0
        elif (
            patient_log.conditions.filter(name__in=[HIV]).exclude(name__in=[DM, HTN]).exists()
        ):
            hiv += 1.0
    if not ncd or not hiv:
        ratio = 0.0
    else:
        ratio = ncd / hiv
    if raise_on_outofrange and not (2.0 <= ratio <= 2.7):
        raise PatientGroupRatioError(
            f"Ratio NDC:HIV not met. Expected at least 2:1. Got {int(ncd)}:{int(hiv)}. "
            "Perhaps catch this in the form."
        )
    ncd = int(ncd)
    hiv = int(hiv)
    ratio = Decimal(ratio)
    return ncd, hiv, ratio


def get_assignment_for_patient_group(group_identifier: str | None) -> str:
    try:
        description = get_assignment_for_subject(
            group_identifier, randomizer_name="default", identifier_fld="group_identifier"
        )
    except SubjectNotRandomization:
        raise PatientGroupNotRandomized("Group is not randomized")
    return description


def get_assignment_description_for_patient_group(group_identifier: str | None) -> str:
    try:
        description = get_assignment_description_for_subject(
            group_identifier, randomizer_name="default", identifier_fld="group_identifier"
        )
    except SubjectNotRandomization:
        raise PatientGroupNotRandomized("Group is not randomized")
    return description


def get_group_subject_dashboards_url(patient_group: PatientGroup | None) -> str | None:
    """Returns an url to the listboard of subjects in followup
    for this group.
    """
    url = None
    randomizer = site_randomizers.get("default")

    if patient_group:
        for obj in randomizer.model_cls().objects.filter(
            group_identifier=patient_group.group_identifier
        ):
            if obj.assignment == COMMUNITY_ARM:
                url = reverse("intecomm_dashboard:subject_listboard_url")
            break
        if not url:
            url = reverse("intecomm_dashboard:subject_listboard_url")
        return f"{url}?q={patient_group.group_identifier}"
    return None


def add_subjects_to_group(group_name: str, subject_identifiers: list[str]):
    """Add subjects to a group after the group has randomized.

    Use with caution.

    The update will work for a participant if:
        * The patient group exists and has been randomized
        * The participant to be added is NOT already in a patient
          group.
        * The participant to be added has been entered into the
          PatientLog, screened and consented.
    """
    try:
        patient_group = PatientGroup.objects.get(name=group_name)
    except ObjectDoesNotExist:
        print(f"PatientGroup does not exist. Got {group_name}.")
    else:
        status = patient_group.status
        for subject_identifier in subject_identifiers:
            try:
                updater = PatientGroupUpdater(patient_group, subject_identifier)
            except PatientGroupUpdaterError as e:
                print(f"    - skipping: {e}")
            except PatientGroupNotRandomized as e:
                print(f"{e}")
                break
            else:
                try:
                    updater.add_subject_to_group()
                except PatientGroupUpdaterError as e:
                    print(f"   - failed: {e}")
                else:
                    print(f"    - added {subject_identifier}")
        patient_group.status = status
        patient_group.save()



# Generated by Django 4.2.3 on 2023-08-07 22:57

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_subject",
            "0094_alter_healtheconomicsproperty_land_surface_area_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="healtheconomicsassets",
            name="bedrooms",
            field=models.IntegerField(
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(30),
                ],
                verbose_name="How many rooms are used for sleeping in your dwelling?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsassets",
            name="beds",
            field=models.IntegerField(
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(30),
                ],
                verbose_name="How many beds does your dwelling have in total?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsassets",
            name="rooms",
            field=models.IntegerField(
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(30),
                ],
                verbose_name="How many rooms does your dwelling have in total, without counting the bathrooms/ toilets or hallways/passageways?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsassets",
            name="bedrooms",
            field=models.IntegerField(
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(30),
                ],
                verbose_name="How many rooms are used for sleeping in your dwelling?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsassets",
            name="beds",
            field=models.IntegerField(
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(30),
                ],
                verbose_name="How many beds does your dwelling have in total?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsassets",
            name="rooms",
            field=models.IntegerField(
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(30),
                ],
                verbose_name="How many rooms does your dwelling have in total, without counting the bathrooms/ toilets or hallways/passageways?",
            ),
        ),
    ]



from django import forms
from edc_consent.modelform_mixins import ConsentModelFormMixin
from edc_form_validators import FormValidatorMixin
from edc_screening.utils import is_eligible_or_raise
from edc_sites.forms import SiteModelFormMixin
from intecomm_form_validators.consent import SubjectConsentFormValidator

from intecomm_screening.utils import raise_if_consent_refusal_exists


class SubjectConsentModelFormMixin(
    SiteModelFormMixin,
    FormValidatorMixin,
    ConsentModelFormMixin,
):
    form_validator_cls = SubjectConsentFormValidator

    screening_identifier = forms.CharField(
        label="Screening identifier",
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )

    def clean(self):
        raise_if_consent_refusal_exists(
            self.cleaned_data.get("screening_identifier"), is_modelform=True
        )
        return super().clean()

    def validate_guardian_and_dob(self):
        """Override method from modelform"""
        pass

    def clean_guardian_and_dob(self):
        """Override method from form validator"""
        pass

    def validate_is_eligible_or_raise(self) -> None:
        screening_identifier = self.get_field_or_raise(
            "screening_identifier", "Screening identifier is required."
        )
        is_eligible_or_raise(
            screening_identifier=screening_identifier,
            url_name="intecomm_screening_admin:intecomm_screening_patientlog_changelist",
        )

    class Meta:
        fields = "__all__"
        help_texts = {
            "identity": (
                "Use Country ID Number, Passport number, driver's license "
                "number or Country ID receipt number"
            ),
            "witness_name": (
                "Required only if participant is illiterate. "
                "Format is 'LASTNAME, FIRSTNAME'. "
                "All uppercase separated by a comma."
            ),
        }
        widgets = {
            "legal_name": forms.TextInput(attrs={"readonly": "readonly"}),
            "familiar_name": forms.TextInput(attrs={"readonly": "readonly"}),
            "initials": forms.TextInput(attrs={"readonly": "readonly"}),
            "identity": forms.TextInput(attrs={"readonly": "readonly"}),
        }



# Generated by Django 4.1.2 on 2022-12-01 03:01

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_consent", "0004_alter_historicalsubjectconsent_familiar_name_and_more"),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name="subjectconsent",
            name="unique_consent_subject_id_and_version",
        ),
        migrations.RemoveConstraint(
            model_name="subjectconsent",
            name="unique_consent_subject_id_screening_id",
        ),
        migrations.RemoveConstraint(
            model_name="subjectconsent",
            name="unique_consent_name_dob_initials",
        ),
        migrations.AlterUniqueTogether(
            name="subjectconsent",
            unique_together={
                ("subject_identifier", "version"),
                ("familiar_name", "dob", "initials", "version"),
                ("subject_identifier", "screening_identifier"),
            },
        ),
    ]



# Generated by Django 4.2.3 on 2023-08-02 23:56

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_lists",
            "0006_consentrefusalreasons_screeningrefusalreasons_and_more",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="arvdrugs",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="arvregimens",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="clinicalwithdrawalreasons",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="clinicservices",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="conditions",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="consentrefusalreasons",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="dmmanagement",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="dmtreatments",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="drugdispensaries",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="drugdispensers",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="drugpaysources",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="healthadvisors",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="healthfacilitytypes",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="healthinterventiontypes",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="healthservices",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="healthtalkconditions",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="healthtalktypes",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="htnmanagement",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="htntreatments",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="laboratorytests",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="locationtypes",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="nonadherencereasons",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="offstudyreasons",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="refillconditions",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="rxmodificationreasons",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="rxmodifications",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="screeningrefusalreasons",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="subjectvisitmissedreasons",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="toxicitywithdrawalreasons",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="transferreasons",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="transportchoices",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="visitreasons",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
    ]



from django.contrib import admin
from edc_constants.constants import OPTION_RETIRED
from edc_he.choices import EXTERNAL_WALL_MATERIALS_CHOICES
from edc_he.modeladmin_mixins import HealthEconomicsAssetsModelAdminMixin

from ...admin_site import intecomm_subject_admin
from ...forms import HealthEconomicsAssetsForm
from ...models import HealthEconomicsAssets
from ..modeladmin_mixins import CrfModelAdmin


@admin.register(HealthEconomicsAssets, site=intecomm_subject_admin)
class HealthEconomicsAssetsAdmin(
    HealthEconomicsAssetsModelAdminMixin,
    CrfModelAdmin,
):
    form = HealthEconomicsAssetsForm

    def get_fieldsets(self, request, obj=None):
        fieldsets = super().get_fieldsets(request, obj=obj)
        if getattr(request, "site", None):
            if request.site.siteprofile.country == "uganda":
                index = 0
                fieldsets = list(fieldsets)
                for label, data in fieldsets:
                    if label == "Household assets (continued)":
                        data["fields"] = tuple([f for f in data["fields"] if f != "dala_dala"])
                        fieldsets[index] = (label, data)
                    index += 1
                fieldsets = tuple(fieldsets)
        return fieldsets

    def formfield_for_choice_field(self, db_field, request, **kwargs):
        if db_field.name == "external_wall_material":
            kwargs["choices"] = self.external_wall_material_choices
        return super().formfield_for_choice_field(db_field, request, **kwargs)

    @property
    def external_wall_material_choices(self) -> tuple[tuple[str, str]]:
        choices: list[tuple[str, str]] = []
        for tpl in EXTERNAL_WALL_MATERIALS_CHOICES:
            if tpl[0] != OPTION_RETIRED:
                choices.append(tpl)
        return tuple(choices)
